#ifndef CONSOLE_H
#define CONSOLE_H

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

namespace Win32Util {

	/**
	 * @class  Console
	 * @brief  Just opens a dynamic console and makes you write to it in colors etc.
	 */
	class Console {

	private:

		// Inited?
		bool   mInited;

		// Console specific variables
		HANDLE mHandle;
		int    mWidth;
		int    mHeight;
		CONSOLE_CURSOR_INFO mCCI;
		
	public:

		// Color scheme, just use Console::black for istance, then calling set Color
		enum Color {
			black       = 0,
			blue        = 1,
			green       = 2,
			cyan        = 3,
			red         = 4,
			purple      = 5,
			brown       = 6,
			gray        = 8,
			light_gray  = 7,
			light_blue  = 9,
			light_green = 10,
			light_cyan  = 11,
			light_red   = 12,
			pink        = 13,
			yellow      = 14,
			white       = 15,
		};
			
		/**
		 * Constuctor
		 */
		Console();

		/**
		 * Destructor
		 */
		~Console();

		/**
		 * Opens a console window
		 *
		 * @param  pWidth  width of buffer
		 * @param  pHeight height of buffer
		 */
		void open(const int pWidth=0, const int pHeight=0);

		/**
		 * Closes the console window
		 */
		void close();
		
		/**
		 * Clears a specific number of rows with the color specified
		 * in setColor(...).
		 *
		 * @param  pRows  Numer of rows (from top) to be cleared.
		 */
		void clear(const int pRows=-1);

		/**
		 * Shows text carret
		 */
		void showCursor();

		/**
		 * Hides text carret
		 */
		void hideCursor();

		/**
		 * Set size of cursor, the height from bottom.
		 *
		 * @param  pSize  Size of carret in percent
		 */
		void setCursorSize(const int pSize=100);

		/**
		 * Sets title of console window
		 */
		void setTitle(const char* pTitle);

		/**
		 * Set color to use with the output commands
		 *
		 * @param  pForegroundColor  Color of text
		 * @param  pBackgroundColor  Color behind the text
		 */
		void setColor(const int pForegroundColor=light_green, const int pBackgroundColor=blue);

		/**
		 * Move cursor (carret) to new position.
		 *
		 * @param  pX  New x (horisontal) position
		 * @param  pY  New y (vertical) position
		 */
		void locate(const int pX=0, const int pY=0);

		/**
		 * Writes a message on screen at carret position.
		 * Note! Carret position is modified by this operation.
		 *
		 * @param  pMessage  The message in prinft kind of syntax
		 */
		void write(const char* pMessage, ...);

		/**
		 * Writes a message on screen at specified position.
		 * Note! Carret position is modified by this operation.
		 *
		 * @param  pX        The x (horisontal) position
		 * @param  pY        The y (vertical) position
		 * @param  pMessage  The message in prinft kind of syntax
		 */
		void write(const int pX, const int pY, const char* pMessage, ...);

	};
}

#endif
